import boto3
import requests
import json

class EsClient:
    def __init__(self, base_url):
        self.base_url = base_url
        self.session = None
        self.headers = None

    def setData(self):
        self.headers = {
            "Content-Type": "application/json"
        }
        self.session = requests.Session()

    def queryConnector(self, connector_name):
        path = "/a_connector/_search"
        query = {"query": { "constant_score": {  "filter": { "bool": { "must": [{ "term": {  "connectorType": "custom" }  },
                  { "term": { "name": connector_name } }, {  "term": {  "orgId": 1  } }  ] } } }  } }
        return self.session.post(self.base_url + path, verify=False, data=json.dumps(query))

class BaseClient:
    def __init__(self, base_url):
        self.base_url = base_url
        self.session = None
        self.headers = None

    def setData(self):
        self.headers = {
            "Content-Type": "application/json"
        }
        self.session = requests.Session()

    def get(self, path):
        return self.session.get(self.base_url + path, verify=False, headers=self.headers)

    def post(self, path, payload=None):
        return self.session.post(self.base_url + path, data=json.dumps(payload), verify=False, headers=self.headers)

    def put(self, path, payload=None):
        return self.session.put(self.base_url + path, data=json.dumps(payload), verify=False, headers=self.headers)

class DynamoDbClient:
    def __init__(self, environment):
        self.environment = environment
        self.dynamo_client = None

    def setClientEnvironment(self):
        if self.environment == 'localdev':
            self.dynamoClient = boto3.client('dynamodb', region_name='us-east-1', endpoint_url='https://access-mockaws:4569', verify=False)
        elif self.environment == 'prod06':
            self.dynamoClient = boto3.client('dynamodb', region_name='eu-central-1')
        else:
            self.dynamoClient = boto3.client('dynamodb')

    def batch_write(self, request):
        return self.dynamoClient.batch_write_item(RequestItems=request, ReturnConsumedCapacity='TOTAL',
                                                  ReturnItemCollectionMetrics='SIZE')
    def put_item(self, table_name, request):
        return self.dynamoClient.put_item(TableName=table_name, Item=request)

    def get_item(self, table_name, key):
        return self.dynamoClient.get_item(TableName=table_name, Key=key)

    def delete_item(self, table_name, key):
        return self.dynamoClient.delete_item(TableName=table_name, Key=key)

    def scan(self, table_name):
        response = self.dynamoClient.scan(TableName=table_name)  # Scan the roles_table whether it's the mock_aws table or a dev/test table
        full_response = None
        # Needed since table can be much larger than boto can grab in one scan
        if 'LastEvaluatedKey' in response:
            full_response = response['Items']
            while 'LastEvaluatedKey' in response:
                response = self.dynamoClient.scan(TableName=table_name, ExclusiveStartKey=response['LastEvaluatedKey'])
                full_response.extend(response['Items'])
                print("Scanning roles table... total roles scanned: ", len(full_response))
            print("Done scanning.")

        return full_response if full_response is not None else response


class PscConnectorClient:
    def __init__(self, base_url, auth_token):
        self.base_url = base_url
        self.auth_token = auth_token
        self.org_id = 1
        self.org_key = None
        self.headers = None
        self.session = None

    def setData(self):
        self.headers = {
            "X-Auth-Token": self.auth_token,
            "Content-Type": "application/json"
        }
        self.session = requests.Session()
        response = self.session.get(
            self.base_url + "/appservices/v5/orgs/{}".format(self.org_id),
            verify=False,
            headers=self.headers
        )

        if not response.ok:
            print('Problem retrieving setting PscConnectorClient Got status code: {}. {}'.format(
                    response.status_code, response.text))
            return response.status_code

        self.org_key = response.json()["organization"]["orgKey"]


    def get(self, path):
        return self.session.get(self.base_url + path, verify=False, headers=self.headers)

    def delete(self, path):
        return self.session.delete(self.base_url + path, verify=False, headers=self.headers)

    def post(self, path, payload=None):
        return self.session.post(self.base_url + path, data=json.dumps(payload), verify=False, headers=self.headers)

    def put(self, path, payload=None):
        return self.session.put(self.base_url + path, data=json.dumps(payload), verify=False, headers=self.headers)